"""Common python utilities for storage providers"""
from charmhelpers.core import hookenv
import os
import subprocess
import sys
import yaml
from yaml.constructor import ConstructorError
import shutil


CRYPTTAB_FILE = "/etc/crypttab"
KEYFILE_DIR = "/root"
CRYPT_MOUNT_SCRIPT = "/usr/local/sbin/crypt_mount.sh"


def log(message, level=None):
    """Quaint little log wrapper for juju logging"""
    hookenv.log(message, level)


# Format a Luks device
def encrypt_device(device_path):
    keyfile = _write_encryption_keyfile(device_path)
    # Create the encrypted volume
    command = ["cryptsetup",
               "--key-file", keyfile,
               "--batch-mode",
               "luksFormat", device_path]
    log("NOTICE: Running: {}".format(command))
    if subprocess.call(command, shell=False) != 0:
        log("ERROR: {} failed".format(command))
        sys.exit(1)
    return get_encrypted_dev_path(device_path)


# Return the Luks device path if it exists
# Perform a LuksOpen if it doesn't
def get_encrypted_dev_path(device_path):
    keyfile = _write_encryption_keyfile(device_path)
    newname = "/dev/mapper/{}".format(os.path.basename(device_path))
    if os.path.exists(newname):
        return newname
    else:
        basedev = os.path.basename(device_path)
        command = ["cryptsetup",
                   "--key-file", keyfile,
                   "open",
                   "--type", "luks",
                   device_path,
                   basedev]
        log("NOTICE: Running: {}".format(command))
        if subprocess.call(command, shell=False) != 0:
            log("ERROR: {} failed".format(command))
            sys.exit(1)
        return newname


def is_encrypted(device_path):
    command = ["cryptsetup", "isLuks", device_path]
    if subprocess.call(command, shell=False) == 0:
        return True
    else:
        return False


def _get_encryption_uuid(device_path):
    command = ["cryptsetup", "luksUUID", device_path]
    try:
        uuid = subprocess.check_output(command, shell=False).rstrip()
        return uuid
    except subprocess.CalledProcessError:
        log("ERROR: {} failed".format(command))
        return None


def get_encryption_key():
    # Get first unit of first data relation
    # (hopefully only unit of only data relation)
    relid = hookenv.relation_ids("data")[0]
    unit = hookenv.related_units(relid)[0]
    log("get_encryption_key() unit: {}".format(unit))
    encryption_key_map = hookenv.config("encryption_key_map")
    encryption_key = None
    if encryption_key_map and unit:
        log("get_encryption_key() Have map and unit")
        try:
            encryption_key_map = yaml.load(encryption_key_map)
            if encryption_key_map:
                encryption_key = encryption_key_map.get(unit, None)
        except ConstructorError as e:
            log("Invalid YAML in 'encryption_key_map': {}".format(e),
                hookenv.WARNING)
            return None
    if not encryption_key:
        log("get_encryption_key() Failed to find key")
    return encryption_key


def _get_encryption_keyfile_name(device_path):
    return "{}/keyfile-{}".format(KEYFILE_DIR, device_path.replace("/", "-"))


# Store the encryption key, as it's needed by the Luks commands
# We will delete it later if the store_encryption_keys config variable
# is not "yes"
def _write_encryption_keyfile(device_path):
    encryption_key = get_encryption_key()
    keyfile = _get_encryption_keyfile_name(device_path)
    # Write out the keyfile
    log("Writing keyfile: {}".format(keyfile))
    with open(keyfile, "w") as f:
        os.chmod(keyfile, 0600)
        f.write(encryption_key)
    return keyfile


def _delete_encryption_keyfile(device_path):
    keyfile = _get_encryption_keyfile_name(device_path)
    if os.path.isfile(keyfile):
        log("Deleting keyfile: {}".format(keyfile))
        os.unlink(keyfile)


# Write the encryption details to the FS to allow auto-mounting
# If store_encryption_keys config option is not "yes", this will
# remove the current device_path from the file
def write_encryption_details(device_path):
    keyfile = _write_encryption_keyfile(device_path)
    basedev = os.path.basename(device_path)
    uuid = _get_encryption_uuid(device_path)
    newCrypttabFile = CRYPTTAB_FILE + ".new"
    if uuid is not None:
        log("Writing encryption details to {}".format(CRYPTTAB_FILE))
        # Read original
        lines = []
        if os.path.exists(CRYPTTAB_FILE):
            with open(CRYPTTAB_FILE, "r") as fr:
                lines = fr.readlines()

        # Write new (removing any existing lines for our device)
        with open(newCrypttabFile, "w") as fw:
            for line in lines:
                if not line.startswith((basedev + " ", basedev + "\t")):
                    fw.write(line)
            if hookenv.config("store_encryption_keys") == "yes":
                fw.write("{} UUID={} {} luks\n".format(basedev, uuid, keyfile))

        # Move into place
        if os.path.exists(CRYPTTAB_FILE):
            os.rename(CRYPTTAB_FILE, CRYPTTAB_FILE + ".orig")
        os.rename(newCrypttabFile, CRYPTTAB_FILE)

        # Remove the keyfile if we don't want it saved
        if hookenv.config("store_encryption_keys") != "yes":
            _delete_encryption_keyfile(device_path)
    else:
        log("Unable to write encryption details to".format(CRYPTTAB_FILE))

    # Add a simple script to help mount the crypted filesystem
    # (only needed is not auto-mounting)
    src = os.path.join(hookenv.charm_dir(), "files", "crypt_mount.sh")
    dest = "/usr/local/sbin/crypt_mount.sh"
    shutil.copyfile(src, dest)
    os.chmod(dest, 0755)
