import crypt_fs
import os
import shutil
import mocker
from testing import TestHookenv


class TestCryptFs(mocker.MockerTestCase):

    def setUp(self):
        self.maxDiff = None
        crypt_fs.hookenv = TestHookenv()
        crypt_fs.KEYFILE_DIR = "/tmp"
        crypt_fs.CRYPTTAB_FILE = "/tmp/crypttab"
        self.addCleanup(setattr, os, "environ", os.environ)
        self.charm_dir = self.makeDir()
        os.environ = {"CHARM_DIR": self.charm_dir}

    def test_get_encryption_key_if_exists(self):
        """
        L{get_encryption_key} returns the encryption key for a
        specific unit
        """
        crypt_fs.hookenv._config = (
            ("encryption_key_map", "{data/0: password1}"),
        )
        self.assertEqual(crypt_fs.get_encryption_key(), "password1")

    def test_get_encryption_key_if_not_exists(self):
        """
        L{get_encryption_key} returns None when no key found
        """
        crypt_fs.hookenv._config = (
            ("encryption_key_map", "{data/1: password1}"),
        )
        self.assertEqual(crypt_fs.get_encryption_key(), None)

    def test_write_encryption_keyfile(self):
        """
        L{_write_encryption_keyfile} writes the key to disk in
        crypt_fs.KEYFILE_DIR
        """
        crypt_fs.hookenv._config = (
            ("encryption_key_map", "{data/0: password1}"),
        )
        keyfile_path = os.path.join(crypt_fs.KEYFILE_DIR,
                                    "keyfile--dev-something")
        if os.path.exists(keyfile_path):
            os.unlink(keyfile_path)
        crypt_fs._write_encryption_keyfile("/dev/something")
        self.assertTrue(os.path.exists(keyfile_path))
        with open(keyfile_path) as infile:
            key = infile.read()
        self.assertEqual(key, "password1")

    def test_delete_encryption_keyfile(self):
        """
        L{_delete_encryption_keyfile} should remove the key from
        crypt_fs.KEYFILE_DIR
        """
        keyfile_path = os.path.join(crypt_fs.KEYFILE_DIR,
                                    "keyfile--dev-something")
        with open(keyfile_path, "w") as outfile:
            outfile.write("test")
        self.assertTrue(os.path.exists(keyfile_path))
        crypt_fs._delete_encryption_keyfile("/dev/something")
        self.assertFalse(os.path.exists(keyfile_path))

    def test_write_encryption_details_no_uuid_no_crypttab(self):
        """
        L{write_encryption_details} Adds a crypttab entry for
        the newly created filesystem.
        It should leave it well alone for this test
        """
        crypt_fs.hookenv._config = (
            ("encryption_key_map", "{data/0: password1}"),
        )

        encryption_uuid = self.mocker.replace(crypt_fs._get_encryption_uuid)
        encryption_uuid("/dev/absent")
        self.mocker.result(None)

        copyfile = self.mocker.replace(shutil.copyfile)
        src = os.path.join(crypt_fs.hookenv.charm_dir(),
                           "files",
                           "crypt_mount.sh")
        copyfile(src, "/usr/local/sbin/crypt_mount.sh")

        chmod = self.mocker.replace(os.chmod)
        chmod("/usr/local/sbin/crypt_mount.sh", 0755)

        self.mocker.replay()

        if os.path.exists(crypt_fs.CRYPTTAB_FILE):
            os.unlink(crypt_fs.CRYPTTAB_FILE)
        self.assertFalse(os.path.exists(crypt_fs.CRYPTTAB_FILE))
        crypt_fs.write_encryption_details("/dev/absent")
        self.assertFalse(os.path.exists(crypt_fs.CRYPTTAB_FILE))

    def test_write_encryption_details_no_uuid_have_crypttab(self):
        """
        L{write_encryption_details} Adds a crypttab entry for
        the newly created filesystem
        It should leave the existing file untouched in this test
        """
        crypt_fs.hookenv._config = (
            ("encryption_key_map", "{data/0: password1}"),
        )

        encryption_uuid = self.mocker.replace(crypt_fs._get_encryption_uuid)
        encryption_uuid("/dev/absent")
        self.mocker.result(None)

        copyfile = self.mocker.replace(shutil.copyfile)
        src = os.path.join(crypt_fs.hookenv.charm_dir(),
                           "files",
                           "crypt_mount.sh")
        copyfile(src, "/usr/local/sbin/crypt_mount.sh")

        chmod = self.mocker.replace(os.chmod)
        chmod("/usr/local/sbin/crypt_mount.sh", 0755)

        self.mocker.replay()

        crypttab_content = "sda_crypt UUID=SOME-EXISTING-UUID none luks\n"
        with open(crypt_fs.CRYPTTAB_FILE, "w") as outfile:
            outfile.write(crypttab_content)
        self.assertTrue(os.path.exists(crypt_fs.CRYPTTAB_FILE))
        crypt_fs.write_encryption_details("/dev/absent")
        with open(crypt_fs.CRYPTTAB_FILE) as infile:
            self.assertEqual(infile.read(), crypttab_content)

    def test_write_encryption_details_have_uuid_no_crypttab(self):
        """
        L{write_encryption_details} Adds a crypttab entry for
        the newly created filesystem
        It should leave the existing file untouched in this test
        """
        crypt_fs.hookenv._config = (
            ("encryption_key_map", "{data/0: password1}"),
            ("store_encryption_keys", "yes"),
        )

        encryption_uuid = self.mocker.replace(crypt_fs._get_encryption_uuid)
        encryption_uuid("/dev/something")
        self.mocker.result("SOME-NEW-UUID")

        copyfile = self.mocker.replace(shutil.copyfile)
        src = os.path.join(crypt_fs.hookenv.charm_dir(),
                           "files",
                           "crypt_mount.sh")
        copyfile(src, "/usr/local/sbin/crypt_mount.sh")

        chmod = self.mocker.replace(os.chmod)
        chmod("/usr/local/sbin/crypt_mount.sh", 0755)

        self.mocker.replay()

        if os.path.exists(crypt_fs.CRYPTTAB_FILE):
            os.unlink(crypt_fs.CRYPTTAB_FILE)
        self.assertFalse(os.path.exists(crypt_fs.CRYPTTAB_FILE))
        crypt_fs.write_encryption_details("/dev/something")
        keyfile_path = os.path.join(crypt_fs.KEYFILE_DIR,
                                    "keyfile--dev-something")
        crypttab_content = (
            "something UUID=SOME-NEW-UUID {} luks\n".format(keyfile_path))
        with open(crypt_fs.CRYPTTAB_FILE) as infile:
            self.assertEqual(infile.read(), crypttab_content)

    def test_write_encryption_details_have_uuid_have_crypttab(self):
        """
        L{write_encryption_details} Adds a crypttab entry for
        the newly created filesystem
        It should add to the file for this test
        """
        crypt_fs.hookenv._config = (
            ("encryption_key_map", "{data/0: password1}"),
            ("store_encryption_keys", "yes"),
        )

        encryption_uuid = self.mocker.replace(crypt_fs._get_encryption_uuid)
        encryption_uuid("/dev/something")
        self.mocker.result("SOME-NEW-UUID")

        copyfile = self.mocker.replace(shutil.copyfile)
        src = os.path.join(crypt_fs.hookenv.charm_dir(),
                           "files",
                           "crypt_mount.sh")
        copyfile(src, "/usr/local/sbin/crypt_mount.sh")

        chmod = self.mocker.replace(os.chmod)
        chmod("/usr/local/sbin/crypt_mount.sh", 0755)

        self.mocker.replay()

        crypttab_content = "sda_crypt UUID=SOME-EXISTING-UUID none luks\n"
        with open(crypt_fs.CRYPTTAB_FILE, "w") as outfile:
            outfile.write(crypttab_content)
        self.assertTrue(os.path.exists(crypt_fs.CRYPTTAB_FILE))
        crypt_fs.write_encryption_details("/dev/something")
        keyfile_path = os.path.join(crypt_fs.KEYFILE_DIR,
                                    "keyfile--dev-something")
        crypttab_content = (
            "sda_crypt UUID=SOME-EXISTING-UUID none luks\n"
            "something UUID=SOME-NEW-UUID {} luks\n".format(keyfile_path))
        with open(crypt_fs.CRYPTTAB_FILE) as infile:
            self.assertEqual(infile.read(), crypttab_content)

    def test_write_encryption_details_have_uuid_have_duplicate_crypttab(self):
        """
        L{write_encryption_details} Adds a crypttab entry for
        the newly created filesystem
        It should replace an existing entry in this test
        """
        crypt_fs.hookenv._config = (
            ("encryption_key_map", "{data/0: password1}"),
            ("store_encryption_keys", "yes"),
        )

        encryption_uuid = self.mocker.replace(crypt_fs._get_encryption_uuid)
        encryption_uuid("/dev/something")
        self.mocker.result("SOME-NEW-UUID")

        copyfile = self.mocker.replace(shutil.copyfile)
        src = os.path.join(crypt_fs.hookenv.charm_dir(),
                           "files",
                           "crypt_mount.sh")
        copyfile(src, "/usr/local/sbin/crypt_mount.sh")

        chmod = self.mocker.replace(os.chmod)
        chmod("/usr/local/sbin/crypt_mount.sh", 0755)

        self.mocker.replay()

        crypttab_content = (
            "sda_crypt UUID=SOME-EXISTING-UUID none luks\n"
            "something UUID=AN-OLD-UUID none luks\n")
        with open(crypt_fs.CRYPTTAB_FILE, "w") as outfile:
            outfile.write(crypttab_content)
        self.assertTrue(os.path.exists(crypt_fs.CRYPTTAB_FILE))
        crypt_fs.write_encryption_details("/dev/something")
        keyfile_path = os.path.join(crypt_fs.KEYFILE_DIR,
                                    "keyfile--dev-something")
        crypttab_content = (
            "sda_crypt UUID=SOME-EXISTING-UUID none luks\n"
            "something UUID=SOME-NEW-UUID {} luks\n".format(keyfile_path))
        with open(crypt_fs.CRYPTTAB_FILE) as infile:
            self.assertEqual(infile.read(), crypttab_content)
